## Overview

[dom_shim.mjs](../dom_shim.mjs) provides a lightweight and performant shim for DOM node and element classes. Allows server-side rendering with [`prax`](prax_readme.md).

## TOC

* [#Usage](#usage)
* [#API](#api)
  * [#Undocumented](#undocumented)

## Usage

```js
import * as ds from 'https://cdn.jsdelivr.net/npm/@mitranim/js@0.1.77/dom_shim.mjs'
```

## API

### Undocumented

The following APIs are exported but undocumented. Check [dom_shim.mjs](../dom_shim.mjs).

  * [`const REF`](../dom_shim.mjs#L12)
  * [`const DATA`](../dom_shim.mjs#L13)
  * [`const NAME`](../dom_shim.mjs#L14)
  * [`const VALUE`](../dom_shim.mjs#L15)
  * [`const STYLE`](../dom_shim.mjs#L16)
  * [`const DOCTYPE`](../dom_shim.mjs#L17)
  * [`const DATASET`](../dom_shim.mjs#L18)
  * [`const PUBLIC_ID`](../dom_shim.mjs#L19)
  * [`const SYSTEM_ID`](../dom_shim.mjs#L20)
  * [`const CLASS_LIST`](../dom_shim.mjs#L21)
  * [`const LOCAL_NAME`](../dom_shim.mjs#L22)
  * [`const ATTRIBUTES`](../dom_shim.mjs#L23)
  * [`const PARENT_NODE`](../dom_shim.mjs#L24)
  * [`const CHILD_NODES`](../dom_shim.mjs#L25)
  * [`const NAMESPACE_URI`](../dom_shim.mjs#L26)
  * [`const OWNER_DOCUMENT`](../dom_shim.mjs#L27)
  * [`const IMPLEMENTATION`](../dom_shim.mjs#L28)
  * [`const DOCUMENT_ELEMENT`](../dom_shim.mjs#L29)
  * [`function isChildNode`](../dom_shim.mjs#L38)
  * [`function reqChildNode`](../dom_shim.mjs#L39)
  * [`function optChildNode`](../dom_shim.mjs#L40)
  * [`function isParentNode`](../dom_shim.mjs#L42)
  * [`function reqParentNode`](../dom_shim.mjs#L43)
  * [`function optParentNode`](../dom_shim.mjs#L44)
  * [`function isNode`](../dom_shim.mjs#L46)
  * [`function reqNode`](../dom_shim.mjs#L47)
  * [`function optNode`](../dom_shim.mjs#L48)
  * [`function isElement`](../dom_shim.mjs#L50)
  * [`function reqElement`](../dom_shim.mjs#L51)
  * [`function optElement`](../dom_shim.mjs#L52)
  * [`function isAttr`](../dom_shim.mjs#L54)
  * [`function reqAttr`](../dom_shim.mjs#L55)
  * [`function optAttr`](../dom_shim.mjs#L56)
  * [`function isText`](../dom_shim.mjs#L58)
  * [`function reqText`](../dom_shim.mjs#L59)
  * [`function optText`](../dom_shim.mjs#L60)
  * [`function isComment`](../dom_shim.mjs#L62)
  * [`function reqComment`](../dom_shim.mjs#L63)
  * [`function optComment`](../dom_shim.mjs#L64)
  * [`function isDocument`](../dom_shim.mjs#L66)
  * [`function reqDocument`](../dom_shim.mjs#L67)
  * [`function optDocument`](../dom_shim.mjs#L68)
  * [`function isDocumentType`](../dom_shim.mjs#L70)
  * [`function reqDocumentType`](../dom_shim.mjs#L71)
  * [`function optDocumentType`](../dom_shim.mjs#L72)
  * [`function isFragment`](../dom_shim.mjs#L74)
  * [`function reqFragment`](../dom_shim.mjs#L75)
  * [`function optFragment`](../dom_shim.mjs#L76)
  * [`function isDomImpl`](../dom_shim.mjs#L78)
  * [`function reqDomImpl`](../dom_shim.mjs#L79)
  * [`function optDomImpl`](../dom_shim.mjs#L80)
  * [`class Node`](../dom_shim.mjs#L84)
  * [`class ElementParent`](../dom_shim.mjs#L287)
  * [`class DocumentFragment`](../dom_shim.mjs#L294)
  * [`class CharacterData`](../dom_shim.mjs#L299)
  * [`class Text`](../dom_shim.mjs#L320)
  * [`class Comment`](../dom_shim.mjs#L327)
  * [`class RawText`](../dom_shim.mjs#L335)
  * [`class Attr`](../dom_shim.mjs#L346)
  * [`class NamedNodeMap`](../dom_shim.mjs#L356)
  * [`class Element`](../dom_shim.mjs#L394)
  * [`class HTMLElement`](../dom_shim.mjs#L601)
  * [`class HTMLMetaElement`](../dom_shim.mjs#L616)
  * [`class HTMLAnchorElement`](../dom_shim.mjs#L621)
  * [`class HTMLLabelElement`](../dom_shim.mjs#L643)
  * [`class HTMLButtonElement`](../dom_shim.mjs#L653)
  * [`class HTMLInputElement`](../dom_shim.mjs#L678)
  * [`class HTMLTextAreaElement`](../dom_shim.mjs#L690)
  * [`class HTMLObjectElement`](../dom_shim.mjs#L692)
  * [`class HTMLOutputElement`](../dom_shim.mjs#L694)
  * [`class HTMLSelectElement`](../dom_shim.mjs#L700)
  * [`class HTMLOptionElement`](../dom_shim.mjs#L708)
  * [`class HTMLFieldSetElement`](../dom_shim.mjs#L716)
  * [`class HTMLFormElement`](../dom_shim.mjs#L718)
  * [`class HTMLTableElement`](../dom_shim.mjs#L722)
  * [`class RawTextElement`](../dom_shim.mjs#L741)
  * [`class HTMLScriptElement`](../dom_shim.mjs#L746)
  * [`class HTMLStyleElement`](../dom_shim.mjs#L747)
  * [`class SVGElement`](../dom_shim.mjs#L754)
  * [`class DocumentType`](../dom_shim.mjs#L759)
  * [`class Document`](../dom_shim.mjs#L783)
  * [`class HTMLDocument`](../dom_shim.mjs#L876)
  * [`class DOMImplementation`](../dom_shim.mjs#L893)
  * [`class CustomElementRegistry`](../dom_shim.mjs#L925)
  * [`class DictPh`](../dom_shim.mjs#L952)
  * [`class StylePh`](../dom_shim.mjs#L965)
  * [`class ClassList`](../dom_shim.mjs#L1084)
  * [`class HTMLFormControlsCollection`](../dom_shim.mjs#L1180)
  * [`class FormControlMap`](../dom_shim.mjs#L1243)
  * [`class RadioNodeList`](../dom_shim.mjs#L1266)
  * [`const global`](../dom_shim.mjs#L1290)
  * [`const document`](../dom_shim.mjs#L1291)
  * [`const customElements`](../dom_shim.mjs#L1292)
  * [`function escapeAttr`](../dom_shim.mjs#L1325)
  * [`function escapeText`](../dom_shim.mjs#L1336)
  * [`function escapeChar`](../dom_shim.mjs#L1342)
  * [`function unescape`](../dom_shim.mjs#L1351)
  * [`function unescapeMatch`](../dom_shim.mjs#L1355)
  * [`class STYLE_TO_CSS`](../dom_shim.mjs#L1370)
  * [`class CAMEL_TO_DATA`](../dom_shim.mjs#L1374)
  * [`class DATA_TO_CAMEL`](../dom_shim.mjs#L1378)
  * [`const OUTER_HTML`](../dom_shim.mjs#L1383)
