import * as l from '../lang.mjs'
import * as cl from '../cli.mjs'
import * as t from '../test.mjs'

Error.stackTraceLimit = Infinity

export const cli = cl.Flag.os()
export const run = cli.get(`--run`)
export const verb = cli.boolOpt(`--verb`)
export const more = cli.boolOpt(`--more`)
export const prec = cli.boolOpt(`--prec`)
export const once = cli.boolOpt(`--once`)
export const clear = cli.boolOpt(`--clear`)

if (clear) cl.emptty()

t.conf.setTestFilter(run)
t.conf.setBenchFilter(run)

if (verb) t.conf.testRep = t.ConsoleStartReporter.default()

// Allows to bench code in "slow mode", without much warmup.
if (once) t.conf.benchRunner = new t.CountRunner(1)

// Opt-in for benchmarks that require more precision than whole nanoseconds.
if (prec) t.conf.benchRep = t.ConsoleAvgReporter.with(t.tsPico)

// Indicates benchmark accuracy. Should be single digit nanoseconds, ideally 0.
t.bench(function bench_baseline() {})

// See comment on `readFull`.
export async function testStream(src, exp) {t.is(await readFull(src), exp)}

/*
Defined in this file because it's used by multiple tests, and we don't want
this anywhere in the public API. This is useful for testing, but violates
the purpose of streams and should not be encouraged. Might move to `test.mjs`.
*/
export async function readFull(src) {
  let out = ``
  for await (const val of src) out += l.reqStr(chunkToStr(val))
  return out
}

export function readFullOpt(src) {
  if (l.isNil(src)) return ``
  return readFull(src)
}

function chunkToStr(val) {
  if (l.isInst(val, Uint8Array)) return new TextDecoder().decode(val)
  return l.reqStr(val)
}
